package es.kauron.jstudy.controller;

import es.kauron.jstudy.model.AppPrefs;
import es.kauron.jstudy.model.TestItem;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Insets;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;
import javafx.scene.control.TextField;
import javafx.scene.layout.HBox;

import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

public class TestController implements Initializable {
    @FXML
    private Label question, prevQuestion, prevAnswer, correctAnswer, progressLabel, correctLabel;
    @FXML
    private TextField answer;
    @FXML
    private ProgressBar progress;
    @FXML
    private HBox feedback;
    @FXML
    private Button skipButton;

    private SimpleBooleanProperty correctingError = new SimpleBooleanProperty(false);
    private List<TestItem> list;
    private int total, current = -1;
    private int done = 0, errors = 0;

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        skipButton.disableProperty().bind(correctingError);
        feedback.visibleProperty().bind(AppPrefs.showFeedback);
    }

    void setList(List<TestItem> list) {
        this.list = list;
        total = list.size();
        progress.setProgress(0);
        progressLabel.setText("0/" + total);
        onSkipAction(null);
    }

    @FXML
    private void onNextAction(ActionEvent event) {
        prevQuestion.setText(list.get(current).getQuestion());
        prevAnswer.setText(answer.getText());
        correctAnswer.setText(list.get(current).getAnswer());

        boolean right = list.get(current).getAnswer().equals(answer.getText());
        correctAnswer.setVisible(!right);
        correctLabel.setVisible(!right);

        if (!right) {
            errors++;
            prevAnswer.setStyle("-fx-text-fill: #C40000;");
        } else {
            prevAnswer.setStyle("-fx-text-fill: #00C400;");
        }

        // Remove the question from the pool if the question was correctly answered or there is no repetition
        if ((right && !correctingError.get()) || !AppPrefs.repeatWrong.get()) {
            list.remove(current);
            progress.setProgress(++done / (double) total);
            progressLabel.setText(done + "/" + total);
            if (list.size() == 0) {
                onEndAction(null);
                return;
            }
            chooseQuestion();
        } else if (!AppPrefs.repeatImmediately.get()) {
            chooseQuestion();
        }
        correctingError.set(!right && AppPrefs.repeatImmediately.get());
        setQuestion();
    }

    private void chooseQuestion() {
        answer.setText("");
        int previous = current;
        do
            current = (int) (Math.random() * list.size());
        while (list.size() > 1 && current == previous);
    }

    private void setQuestion() {
        answer.setText("");
        question.setText(list.get(current).getQuestion());
        answer.requestFocus();
    }

    @FXML
    private void onSkipAction(ActionEvent event) {
        chooseQuestion();
        setQuestion();
    }

    @FXML
    private void onEndAction(ActionEvent event) {
        answer.setText("");
        answer.setDisable(true);
        question.setText("That's it!");
        feedback.getChildren().clear();
        feedback.setSpacing(5);
        feedback.setPadding(new Insets(5, 5, 5, 5));
        feedback.getChildren().add(new Label("Mistakes: " + errors));
        feedback.getChildren().add(new Label(String.format("Mark: %.2f%%", (total - errors) / (double) total * 100)));
    }
}
